@@ -27,7 +27,6 @@ struct object* new_queue(){
 void free_queue(struct object* obj) {
 	munmap(obj->q, obj->fil_len);
 	close(obj->fd);
-	//free(obj->file);
 	free(obj);
 }
 void calc_queue(struct object* obj,const char* file,int que_len,int rec_len){
@@ -37,9 +36,7 @@ void calc_queue(struct object* obj,const char* file,int que_len,int rec_len){
 		rec_len += 4 - pad;
 	}
 	obj->rec_len = rec_len;
-
 	obj->que_len = que_len;
-
 	obj->fil_len = (sizeof(struct record) + rec_len) * que_len + sizeof(struct queue);
 	if( (pad = obj->fil_len % 4096)){
 		obj->fil_len += 4096 - pad;
@@ -137,37 +134,7 @@ void push_queue(struct object* obj,const char* value,int len){
 	obj->q->top = top;
 	unlock_queue(obj);
 }
-int pop_queue(struct object* obj,char** value,int* len){
-	int cur,top,first,size;
-	lock_queue(obj);
-	cur = obj->q->bottom;
-	first = top = obj->q->top;
-	if(cur == top){
-		unlock_queue(obj);
-		return 0;
-	}
-	*len = 0;
-	do {
-		if(++top >= obj->que_len){
-			top = 0;
-		}
-		*len += obj_list(obj,top)->len;
-	} while(! obj_list(obj,top)->last);
-	*value = (char*)malloc(*len);
-	top = first;
-	first = 0;
-	do {
-		if(++top >= obj->que_len){
-			top = 0;
-		}
-		size = obj_list(obj,top)->len;
-		memcpy((*value) + first,obj_list(obj,top)->data,size);
-	} while((first += size) != *len);
-	obj->q->top = top;
-	unlock_queue(obj);
-	return 1;
-}
-SV* pop_queue_2(struct object* obj){
+SV* pop_queue(struct object* obj){
 	int cur,top,first,size,len;
 	SV* value;
 	lock_queue(obj);
@@ -199,6 +166,39 @@ SV* pop_queue_2(struct object* obj){
 	unlock_queue(obj);
 	return value;
 }
+SV* top_queue(struct object* obj){
+	int cur,top,first,size,len;
+	SV* value;
+
+	//lock_queue(obj);
+	cur = obj->q->bottom;
+	first = top = obj->q->top;
+	if(cur == top){
+		//unlock_queue(obj);
+		return 0;
+	}
+	len = 0;
+	do {
+		if(++top >= obj->que_len){
+			top = 0;
+		}
+		len += obj_list(obj,top)->len;
+	} while(! obj_list(obj,top)->last);
+	value = newSVpvn("",0);
+	SvGROW(value,len);
+	top = first;
+	first = 0;
+	do {
+		if(++top >= obj->que_len){
+			top = 0;
+		}
+		size = obj_list(obj,top)->len;
+		sv_catpvn(value, obj_list(obj,top)->data, (STRLEN)size);
+	} while((first += size) != len);
+	//obj->q->top = top;
+	//unlock_queue(obj);
+	return value;
+}
 void unlock_queue(struct object* obj){
 	struct flock lock;
 	
@@ -258,6 +258,6 @@ int lock_queue(struct object* obj){
 	
 	return obj->locked = 1;
 }
-struct record* obj_list(struct object* obj,int i) {
-	return (struct record*)(obj->q->list + (sizeof(struct record) + obj->rec_len)*i);
-}
+//struct record* obj_list(struct object* obj,int i) {
+//	return (struct record*)(obj->q->list + (sizeof(struct record) + obj->rec_len)*i);
+//}
